#pragma rtGlobals=1		// Use modern global access method.
//-----------------------------------
// created 20/08/2012 by T.O.Mentes
//
// Based on the LEED_FIT.ipf routine
//           - handling of different types of peaks improved
//           - data display improved 
//
//-----------------------------------

Menu "GALO"
	"GALO_initialize"
	"GALO_cleanup"
	"GALO_backup"
	"GALO_recallBackup"
	"GALO_singleFit"
	"GALO_seriesFit"
	"GALO_parseResults"
	"GALO_WF_seriesFit"
End

//---------------------------------------------------------------------------------
//--- Initialize the global waves which hold the fit parameters
Macro GALO_initialize(numpeakstmp)
	variable numpeakstmp=1
	Prompt numpeakstmp, "Total number of peaks to fit (max 9): "

	variable/G numpeaks = numpeakstmp
	
  if (numpeaks>9)
    numpeaks=9
  endif
	
  make/n=40/o w_coef          // create the coefficient waves
	w_coef[]=(!(mod(p,4)==1))   // set amplitudes to zero
	String/G holdStr="1111111111111111111111111111111111111111"

	make/n=10/O/T prf_shape
	make/n=10/O prf_amp=nan
	make/n=10/O prf_width=nan
	make/n=10/O prf_pos=nan
	make/n=10/O prf_asym=nan
	make/n=(10)/O/T prf_hold
	make/n=(1)/O/T prf_constraint
	make/n=1/O prf_wavex
	make/n=1/O prf_wavey
	prf_constraint = "1>0"

  // initialize background
  prf_shape[0] = "B"
  prf_amp[0] = 0
  prf_width[0] = 0
  prf_pos[0] = 0
  prf_asym[0] = 0
  prf_hold[0] = "0111"
  holdStr[0,3] = prf_hold[0]
  
  edit prf_shape, prf_amp, prf_width, prf_pos, prf_asym, prf_hold, prf_constraint
	DoWindow/C GALO_table
	DoWindow/T GALO_table,"GALO: fit parameters"
	display prf_wavey vs prf_wavex
	ModifyGraph mode=3,marker=19
	DoWindow/C GALO_plot
	DoWindow/T GALO_plot,"GALO: fit plot"
  
  String tmpWaveStr
  variable peakcounter=1
  do
    if (peakcounter <= numpeaks)
      prf_shape[peakcounter] = "G"
      prf_hold[peakcounter] = "0001"
      holdStr[peakcounter*4,peakcounter*4+3] = prf_hold[peakcounter]
      tmpWaveStr = "fit_peak"+num2str(peakcounter)
      make/n=1/o $tmpWaveStr
      appendToGraph $tmpWaveStr vs prf_wavex
      ModifyGraph rgb($tmpWaveStr)=(0,15872,65280)
    else
      prf_shape[peakcounter] = ""
      prf_hold[peakcounter] = "1111"
      holdStr[peakcounter*4,peakcounter*4+3] = prf_hold[peakcounter]
    endif
    
    peakcounter+=1
  while (peakcounter<10)

end


//---------------------------------------------------------------------------------
// --- number of peaks, type of fits, and fit constraints are initialized already
// --- Go ahead with fitting procedures.
//------------------------------
Macro GALO_singleFit(waveXStr,waveYStr)
	String waveXStr
	Prompt waveXStr, "enter X wave",popup,"calculated;"+WaveList("*",";","WIN:")
	String waveYStr
	Prompt waveYStr, "enter Y wave",popup,WaveList("*",";","WIN:")
	
	variable pStart
	variable pStop
	
	abortFlag=0
	silent 1
	
	if ((strlen(CsrInfo(A))==0) || (strlen(CsrInfo(B))==0))
		print "Cursors are not on the graph, setting to full range!!"
		pStart = 0
		pStop = numPnts($waveYStr)-1
	else
		pStart = pcsr(A)
		pStop = pcsr(B)
	endif
	
	if (pStart >pStop)
		print "Select the range by placing the cursors on the plot. Cursor-A comes first!!"
		abortFlag=1
	endif
	
	if( !WaveExists(prf_shape) )
		print "First run the initialize routine!!"
		abortFlag=1
	endif
	
	if (abortFlag==0)
		// generate the new constraint array
		duplicate/o prf_constraint, prf_constraintK
		GALO_interpretConstraint(prf_constraintK)
	
		variable abortFlag=0
		variable peakCounter
		String tempWaveStr
		variable numDataPts = pStop - pStart
	
		Redimension/N=(numDataPts) prf_wavex
		Redimension/N=(numDataPts) prf_wavey
		if (stringmatch(waveXStr,"calculated"))
			prf_wavex = x	+ pStart
		else
			prf_wavex[] = $waveXStr[p+pStart]
		endif
		prf_wavey[] = $waveYStr[p+pStart]
		peakCounter=0
		do
			w_coef[4*peakCounter] = prf_amp[peakCounter]
			w_coef[4*peakCounter+1] = prf_width[peakCounter]
			w_coef[4*peakCounter+2] = prf_pos[peakCounter]
			w_coef[4*peakCounter+3] = prf_asym[peakCounter]
			holdStr[4*peakCounter,4*peakCounter+3] = prf_hold[peakCounter]
			peakCounter+=1
		while(peakCounter<=numpeaks)
		
		print holdStr
		print w_coef
		
    		DoWindow/F GALO_plot
		FuncFit/Q/H=holdStr fitGALOProfile w_coef prf_wavey /X=prf_wavex /D /C=prf_constraintK	

		peakCounter=0
		do
			prf_amp[peakCounter] = w_coef[4*peakCounter]
			prf_width[peakCounter] = w_coef[4*peakCounter+1]
			prf_pos[peakCounter] = w_coef[4*peakCounter+2]
			prf_asym[peakCounter] = w_coef[4*peakCounter+3]
			
			if (peakCounter>0)
			 tempWaveStr = "fit_peak"+num2str(peakCounter)
			 Redimension/N=(numDataPts) $tempWaveStr
			 if (stringmatch(prf_shape[peakCounter],"G"))
			   $tempWaveStr = prf_amp[peakCounter] * exp(-0.693147*(prf_wavex-prf_pos[peakCounter])^2/(prf_width[peakCounter])^2) / (1+exp((prf_pos[peakCounter]-prf_wavex)/prf_asym[peakCounter]))
			 endif
			 if (stringmatch(prf_shape[peakCounter],"L"))
			   $tempWaveStr = prf_amp[peakCounter] * (prf_width[peakCounter])^2 / ((prf_wavex-prf_pos[peakCounter])^2 + (prf_width[peakCounter])^2)
			 endif
			endif
			
			peakCounter+=1
		while(peakCounter<=numpeaks)
		
		print "The chisquare is: ",V_chisq
	endif
	
end

//-------------------------------------------------------------------------------------------------------------------------------
Macro GALO_seriesFit(waveXStr, coefNameSuffix)
	String waveXStr
	Prompt waveXStr, "enter X wave",popup,"calculated;"+WaveList("*P",";","WIN:")
	String coefNameSuffix
	Prompt coefNameSuffix, "Lutfen katsayi ekini giriniz"
	
	String waveNamesStr = WaveList("*",";","WIN:")
	variable numProfiles = ItemsInList(waveNamesStr)
	String resultWaveStr = "prf_results_"+coefNameSuffix
	if (exists(resultWaveStr) != 1)
		make/o/N=(numProfiles,41) $resultWaveStr
	endif
	String errorWaveStr = "prf_errors_"+coefNameSuffix
	if (exists(errorWaveStr) != 1)
		make/o/N=(numProfiles,41) $errorWaveStr
	endif
	
	silent 1
	variable abortFlag=0
	variable peakCounter
	variable numDataPts
	String waveYStr
	String tempWaveStr
	
	variable pStart
	variable pStop
	pStart = pcsr(A)
	pStop = pcsr(B)
	if (pStart >pStop)
		print "Select the range by placing the cursors on the plot. Cursor-A comes first!!"
		abortFlag=1
	endif
	
	// generate the new contraint array
	duplicate/o prf_constraint, prf_constraintK
	GALO_interpretConstraint(prf_constraintK)
	
	waveYStr = StringFromList(0, waveNamesStr)
	numDataPts = pStop - pStart
	Redimension/N=(numDataPts) prf_wavex
	Redimension/N=(numDataPts) prf_wavey
	//prf_wavex = x + pStart
	if (stringmatch(waveXStr,"calculated"))
		prf_wavex = x	+ pStart
	else
		prf_wavex[] = $waveXStr[p+pStart]
	endif

	if( !WaveExists(prf_shape) )
		print "First run the initialize routine!!"
		abortFlag=1
	endif
	
	if (abortFlag==0)
	  	peakCounter=0
		do
			w_coef[4*peakCounter] = prf_amp[peakCounter]
			w_coef[4*peakCounter+1] = prf_width[peakCounter]
			w_coef[4*peakCounter+2] = prf_pos[peakCounter]
			w_coef[4*peakCounter+3] = prf_asym[peakCounter]
			holdStr[4*peakCounter,4*peakCounter+3] = prf_hold[peakCounter]
			peakCounter+=1
		while(peakCounter<=numpeaks)

		variable i=0
		do
			waveYStr = StringFromList(i, waveNamesStr)
			//print waveYStr
			prf_wavey[] = $waveYStr[p+pStart]
			
			if (!stringmatch(waveYStr,"*_P"))
			
				DoWindow/F GALO_plot
				FuncFit/Q/H=holdStr fitGALOProfile w_coef prf_wavey /X=prf_wavex /D /C=prf_constraintK	

				//holdStr[0] = "1"  // fix the background after first spectrum

		 		peakCounter=0
		 	 	do
				 	prf_amp[peakCounter] = w_coef[4*peakCounter]
				 	prf_width[peakCounter] = w_coef[4*peakCounter+1]
				 	prf_pos[peakCounter] = w_coef[4*peakCounter+2]
				 	prf_asym[peakCounter] = w_coef[4*peakCounter+3]
				 	if (peakCounter>0)
					 	tempWaveStr = "fit_peak"+num2str(peakCounter)
					 	Redimension/N=(numDataPts) $tempWaveStr
					 	if (stringmatch(prf_shape[peakCounter],"G"))
					   		$tempWaveStr = prf_amp[peakCounter] * exp(-0.693147*(prf_wavex-prf_pos[peakCounter])^2/(prf_width[peakCounter])^2) / (1+exp((prf_pos[peakCounter]-prf_wavex)/prf_asym[peakCounter]))
						endif
					 	if (stringmatch(prf_shape[peakCounter],"L"))
					   		$tempWaveStr = prf_amp[peakCounter] * (prf_width[peakCounter])^2 / ((prf_wavex-prf_pos[peakCounter])^2 + (prf_width[peakCounter])^2)
					 	endif
					 endif
			 		peakCounter+=1
		  		while(peakCounter<=numpeaks)
			
				peakCounter=0
				do
					$resultWaveStr[i][peakCounter] = w_coef[peakCounter]
					$errorWaveStr[i][peakCounter] = w_sigma[peakCounter]
					peakCounter+=1
				while(peakCounter<40)
				$resultWaveStr[i][40] = V_chisq
			endif
			
      			i+=1
		while (i<numProfiles)
	
  endif
	
end

//-------------------------------------------------------------------------------------------------------------------------------
Macro GALO_WF_seriesFit(coefNameSuffix)
	String coefNameSuffix
	Prompt coefNameSuffix, "Lutfen katsayi ekini giriniz"
	
	// extract the waterfall name
	String WF_nameStr = StringFromList(0,WaveList("*_WF",";","WIN:"))
	String WFP_nameStr = StringFromList(0,WaveList("*_P",";","WIN:"))
	String WFQ_nameStr = StringFromList(0,WaveList("*_Q",";","WIN:"))
	variable abortFlag = 0
	if (strlen(WF_nameStr)==0)
		abortFlag = 1
		print "Select the waterfall window first."
	endif
	
	if ( (strlen(csrinfo(A))==0) || (strlen(csrinfo(A))==0) )
		abortFlag = 1
		print "Place both cursors on the waterfall window first (A bottom left, B top right)."
	endif
	
	if( !WaveExists(prf_shape) )
		print "First run the initialize routine!!"
		abortFlag=1
	endif
	
	// before running the fitting routine, initialize parameters, result arrays, etc...
	if (abortFlag == 0)
		variable numProfiles = dimsize($WF_nameStr, 1)
	
		String resultWaveStr = "prf_results_"+coefNameSuffix
		String errorWaveStr = "prf_errors_"+coefNameSuffix
		if (exists(resultWaveStr) != 1)
			make/o/N=(numProfiles,41) $resultWaveStr
			$resultWaveStr = NaN
		endif
		if (exists(errorWaveStr) != 1)
			make/o/N=(numProfiles,41) $errorWaveStr
			$errorWaveStr = NaN
		endif
	
		variable peakCounter
		variable numDataPts
		String waveYStr
		String tempWaveStr
	
		variable prfStart
		variable prfStop
		prfStart = qcsr(A)
		prfStop = qcsr(B)
		variable pStart
		variable pStop
		pStart = pcsr(A)
		pStop = pcsr(B)
		
		if ( pStart >pStop)
			print "Cursor A: BOTTOM LEFT,        Cursor B: TOP RIGHT"
			abortFlag=1
		else	
			// generate the new contraint array
			duplicate/o prf_constraint, prf_constraintK
			GALO_interpretConstraint(prf_constraintK)
	
			numDataPts = pStop - pStart
			Redimension/N=(numDataPts) prf_wavex
			Redimension/N=(numDataPts) prf_wavey
			//prf_wavex = x + pStart
			prf_wavex[] = $WFP_nameStr[p+pStart]
		endif
		
		variable scanDirection = 1
		if (prfStart > prfStop)
			scanDirection = -1
		endif

	endif
	
	if (abortFlag==0)
	  	peakCounter=0
		do
			w_coef[4*peakCounter] = prf_amp[peakCounter]
			w_coef[4*peakCounter+1] = prf_width[peakCounter]
			w_coef[4*peakCounter+2] = prf_pos[peakCounter]
			w_coef[4*peakCounter+3] = prf_asym[peakCounter]
			holdStr[4*peakCounter,4*peakCounter+3] = prf_hold[peakCounter]
			peakCounter+=1
		while(peakCounter<=numpeaks)

		variable i=prfStart
		do
			prf_wavey[] = $WF_nameStr[p+pStart][i]
			
			DoWindow/F GALO_plot
			FuncFit/Q/H=holdStr fitGALOProfile w_coef prf_wavey /X=prf_wavex /D /C=prf_constraintK		
			//holdStr[0] = "1"  // fix the background after first spectrum

		 	 peakCounter=0
		 	 do
			 	prf_amp[peakCounter] = w_coef[4*peakCounter]
			 	prf_width[peakCounter] = w_coef[4*peakCounter+1]
			 	prf_pos[peakCounter] = w_coef[4*peakCounter+2]
			 	prf_asym[peakCounter] = w_coef[4*peakCounter+3]
			 	if (peakCounter>0)
				 	tempWaveStr = "fit_peak"+num2str(peakCounter)
				 	Redimension/N=(numDataPts) $tempWaveStr
				 	if (stringmatch(prf_shape[peakCounter],"G"))
				   		$tempWaveStr = prf_amp[peakCounter] * exp(-0.693147*(prf_wavex-prf_pos[peakCounter])^2/(prf_width[peakCounter])^2) / (1+exp((prf_pos[peakCounter]-prf_wavex)/prf_asym[peakCounter]))
					endif
				 	if (stringmatch(prf_shape[peakCounter],"L"))
				   		$tempWaveStr = prf_amp[peakCounter] * (prf_width[peakCounter])^2 / ((prf_wavex-prf_pos[peakCounter])^2 + (prf_width[peakCounter])^2)
				 	endif
				 endif
			 	peakCounter+=1
		  	while(peakCounter<=numpeaks)
			
			peakCounter=0
			do
				$resultWaveStr[i][peakCounter] = w_coef[peakCounter]
				$errorWaveStr[i][peakCounter] = w_sigma[peakCounter]
				peakCounter+=1
			while(peakCounter<40)
			$resultWaveStr[i][40] = V_chisq
			
      			i+=scanDirection
		while (scandirection*i < scanDirection*prfStop)
	
  endif
	
end

//---------------------------------------------------------------------------------
//---- The following are the auxiliary functions
//---------------------------------------------------------------------------------
Macro GALO_cleanup()
	funcGALO_cleanup()
end

function funcGALO_cleanup()
	DoWindow/K GALO_table
	DoWindow/K GALO_plot
	killwaves prf_shape
	killwaves prf_amp
	killwaves prf_width
	killwaves prf_pos
	killwaves prf_asym
	killwaves prf_hold
	killwaves prf_constraint
	killwaves prf_wavex
	killwaves prf_wavey
	
	killwaves backup_shape
	killwaves backup_amp
	killwaves backup_width
	killwaves backup_pos
	killwaves backup_asym
	killwaves backup_hold
	killwaves backup_constraint
end

//---------------------------------------------------------------------------------
Macro GALO_backup()
	duplicate/o/T prf_shape, backup_shape
	duplicate/o prf_amp, backup_amp
	duplicate/o prf_width, backup_width
	duplicate/o prf_pos, backup_pos
	duplicate/o prf_asym, backup_asym
	duplicate/o/T prf_hold, backup_hold
	duplicate/o/T prf_constraint, backup_constraint
end

//---------------------------------------------------------------------------------
Macro GALO_recallBackup()
	if ((Exists("backup_shape")*Exists("backup_pos")*Exists("backup_amp")*Exists("backup_width")) == 1)
		prf_shape = backup_shape
		prf_amp = backup_amp
		prf_width = backup_width
		prf_pos = backup_pos
		prf_asym = backup_asym
		prf_hold = backup_hold
		duplicate/o/T backup_constraint, prf_constraint
	else
		print "There is no backup!"
	endif
end

//---------------------------------------------------------------------------------
function GALO_interpretConstraint(newConstraintArray)
	wave/T newConstraintArray
	wave/T w_constraint=prf_constraint
	NVAR numpeaks = numpeaks
	
	variable abortFlag=0
	variable numConstraint = numpnts(w_constraint)
	if (numConstraint != numpnts(newConstraintArray))
		print "GALO_interpretConstraint: check array lengths!!"
		abortFlag=1
	endif
	
	if (abortFlag == 0)
	
		newConstraintArray = ""
		string tmpStr
		variable tmpStrLength
		variable tmpPeakIndex
		variable i, j
		i=0
		do
			tmpStr = w_constraint[i]
			tmpStrLength = strlen(tmpStr)
			
			j=0
			do
				if (stringmatch(tmpStr[j],"A"))
					tmpPeakIndex = str2num(tmpStr[j+1])
					if (tmpPeakIndex > numpeaks)
						j=tmpStrLength
						newConstraintArray[i] = "1>0"
					else
						newConstraintArray[i] += "K"+num2str(4*tmpPeakIndex)
						j=j+2
					endif
				elseif (stringmatch(tmpStr[j],"W"))
					tmpPeakIndex = str2num(tmpStr[j+1])
					if (tmpPeakIndex > numpeaks)
						j=tmpStrLength
						newConstraintArray[i] = "1>0"
					else
						newConstraintArray[i] += "K"+num2str(4*tmpPeakIndex+1)
						j=j+2
					endif
				elseif (stringmatch(tmpStr[j],"P"))
					tmpPeakIndex = str2num(tmpStr[j+1])
					if (tmpPeakIndex > numpeaks)
						j=tmpStrLength
						newConstraintArray[i] = "1>0"
					else
						newConstraintArray[i] += "K"+num2str(4*tmpPeakIndex+2)
						j=j+2
					endif
				else
					newConstraintArray[i] += tmpStr[j]
					j=j+1
				endif
			while(j<tmpStrLength)
			
			i=i+1
		while (i<numConstraint)

	else
		print "GALO_interpretConstraint: aborted..."
	endif

end
//---------------------------------------------------------------------------------
function GALO_addPeak(): GraphMarquee
	wave/T w_shape=prf_shape
	wave w_amp=prf_amp
	wave w_width=prf_width
	wave w_pos=prf_pos
	wave w_asym=prf_asym
	wave/T w_hold=prf_hold
	NVAR numpeaks = numpeaks
	
	variable shapeFlag=1
	Prompt shapeFlag, "Select peak shape?", popup "Gaussian;Lorentzian"
	DoPrompt "GALO_addPeak", shapeFlag
	
	GetMarquee/Z/K left, bottom

	variable abortFlag=0

	if( !WaveExists(w_shape))
		print "First run the initialize routine!"
		abortFlag=1
	endif
	if (V_Flag == 0)
		print "There is no marquee!"
		abortFlag=1
	endif
	if (numpeaks == 9)
		print "Cannot add another peak: enough peaks already!"
		abortFlag=1
	endif
	
	DoWindow/F GALO_plot
	if (abortFlag==0)
		numpeaks += 1
		if (shapeFlag == 1)
			w_shape[numpeaks] = "G"
			w_amp[numpeaks] = abs(V_top-V_bottom)
			w_width[numpeaks] = abs(V_left - V_right)/2
			w_pos[numpeaks] = (V_left + V_right)/2
			w_asym[numpeaks] = 10
			w_hold[numpeaks] = "0001"
		endif
		if (shapeFlag == 2)
			w_shape[numpeaks] = "L"
			w_amp[numpeaks] = abs(V_top-V_bottom)
			w_width[numpeaks] = abs(V_left - V_right)/2
			w_pos[numpeaks] = (V_left + V_right)/2
			w_asym[numpeaks] = 10
			w_hold[numpeaks] = "0001"
		endif
		String tmpWaveStr = "fit_peak"+num2str(numpeaks)
	      make/n=1/o $tmpWaveStr
      		appendToGraph $tmpWaveStr
	endif
	
end

//---------------------------------------------------------------------------------
function GALO_updatePeak(): GraphMarquee
	wave/T w_shape=prf_shape
	wave w_amp=prf_amp
	wave w_width=prf_width
	wave w_pos=prf_pos
	wave w_asym=prf_asym
	NVAR numpeaks = numpeaks
	
	variable peakIndex=1
	Prompt peakIndex, "Which peak to update (index: 1,...,NP)?"
	variable shapeFlag=1
	Prompt shapeFlag, "Select new peak shape?", popup "Gaussian;Lorentzian"
	DoPrompt "GALO_updatePeak", peakIndex, shapeFlag

	//DoWindow/F GALO_plot
	
	GetMarquee/Z/K left, bottom

	variable abortFlag=0
	if( !WaveExists(w_shape))
		print "First run the initialize routine!"
		abortFlag=1
	endif	

	if ((peakIndex > numpeaks) || (peakIndex<1))
			print "Peak index out of range."
			abortFlag=1
	endif

	if (V_Flag == 0)
		print "There is no marquee!"
		abortFlag=1
	endif
	
	if (abortFlag==0)
		if (shapeFlag == 1)
			w_shape[peakIndex] = "G"
			w_amp[peakIndex] = abs(V_top-V_bottom)
			w_width[peakIndex] = abs(V_left - V_right)/2
			w_pos[peakIndex] = (V_left + V_right)/2
			w_asym[peakIndex] = 10
		endif
		if (shapeFlag == 2)
			w_shape[peakIndex] = "L"
			w_amp[peakIndex] = abs(V_top-V_bottom)
			w_width[peakIndex] = abs(V_left - V_right)/2
			w_pos[peakIndex] = (V_left + V_right)/2
			w_asym[peakIndex] = 10
		endif
	endif
end

//---------------------------------------------------------------------------------
Macro GALO_parseResults()
	funcGALO_parseResults()
end

function funcGALO_parseResults()
	string resultSuffix=""
	prompt resultSuffix, "Enter the suffix of the result wave"
	doprompt "funcGALO_parseResults", resultSuffix
	
	wave w_results = $"prf_results_"+resultSuffix
	wave w_errors = $"prf_errors_"+resultSuffix
	wave/T w_shape = prf_shape
	NVAR numpeaks = numpeaks
	
	variable abortFlag=0
	if (!waveexists(w_results) || !waveexists(w_shape))
		print "funcGALO_parseResults: result wave does not exist. ABORTING."
		abortFlag = 1
	else
		variable numRows, numCols
		numRows = dimsize(w_results,0)
		numCols = dimsize(w_results,1)
		if (numCols != 41)
			print  "funcGALO_parseResults: improper number of coefficients. ABORTING."
			abortFlag = 1
		endif
	endif
	
	if (abortFlag == 0)
	
    string parsedWaveStr
    string parsedErrorStr

		variable i=1
		do
		  	// amplitude
		  	parsedWaveStr = resultSuffix+"_peak" + num2str(i) + "_A"
			make/n=(numRows)/o $parsedWaveStr
		 	wave w_parsedWave=$parsedWaveStr
		  	parsedErrorStr = parsedWaveStr+"_e"
		  	make/n=(numRows)/o $parsedErrorStr
			wave w_parsedError=$parsedErrorStr		
			w_parsedWave[] = w_results[p][4*i]
			w_parsedError[] = w_errors[p][4*i]
		  
		  // width
		  parsedWaveStr = resultSuffix+"_peak" + num2str(i) + "_HWHM"
		  make/n=(numRows)/o $parsedWaveStr
		  wave w_parsedWave=$parsedWaveStr
		  parsedErrorStr = parsedWaveStr+"_e"
			make/n=(numRows)/o $parsedErrorStr
			wave w_parsedError=$parsedErrorStr		
			w_parsedWave[] = w_results[p][4*i+1]
			w_parsedError[] = w_errors[p][4*i+1]
			
		  // position
		  parsedWaveStr = resultSuffix+"_peak" + num2str(i) + "_P"
		  make/n=(numRows)/o $parsedWaveStr
		  wave w_parsedWave=$parsedWaveStr
		  parsedErrorStr = parsedWaveStr+"_e"
			make/n=(numRows)/o $parsedErrorStr
			wave w_parsedError=$parsedErrorStr		
			w_parsedWave[] = w_results[p][4*i+2]
			w_parsedError[] = w_errors[p][4*i+2]
			
		// area
		  parsedWaveStr = resultSuffix+"_peak" + num2str(i) + "_area"
		  make/n=(numRows)/o $parsedWaveStr
		  wave w_parsedWave=$parsedWaveStr		
		  w_parsedWave[] = 2 * w_results[p][4*i] * w_results[p][4*i+1]

			i=i+1
		while(i<=numpeaks)

	endif
	
end

//---------------------------------------------------------------------------------
//---- Here is the actual fitting function --------------------------------
//---------------------------------------------------------------------------------
Function fitGALOprofile(w,x) : FitFunc
	WAVE w
	Variable x
	wave/T w_shape=prf_shape
	NVAR numpeaks = numpeaks
	
	//CurveFitDialog/ Equation:
	//CurveFitDialog/ f(x) = y0 + y1*x + y2*x^2 + peak1 + peak2 + ... + peakN
	//CurveFitDialog/ End of Equation
	//CurveFitDialog/ Independent Variables 1
	//CurveFitDialog/ x
	//CurveFitDialog/ Coefficients 31
	//CurveFitDialog/ w[0] = y0
	//CurveFitDialog/ w[1] = y1
	//CurveFitDialog/ w[2] = y2
	//CurveFitDialog/ w[3] = amplitude
	// etc etc...

	variable tmpSum
	// background: polynomial to second order
	tmpSum=w[0] + w[1]*x + w[2]*x^2 + w[3]*x^3
	variable i=1
	do
		if (stringmatch(w_shape[i],"G"))
  			tmpSum += w[4*i] * exp(-0.693147*((x-w[4*i+2])/w[4*i+1])^2) / (1+exp((w[4*i+2]-x)/w[4*i+3]))
  		endif
  		if (stringmatch(w_shape[i],"L"))
      			tmpSum += w[4*i]* w[4*i+1]^2 / ((x-w[4*i+2])^2+w[4*i+1]^2)
    		endif   
		i+=1
	while (i<=numpeaks)
	return  tmpSum

End

