#pragma rtGlobals=3		// Use modern global access method and strict wave access.

//---------------------------------------------
// Created by TOM on 11/01/2019
//---------------------------------------------
Menu "LEED Correction"
	"Determine and Apply DC", LEEDCor_determineDistortion()
	"Apply Existing DC", LEEDCor_correctDistortion()
end

//-------------------------------------------------------------------------------------
// Calculates the distortions from a LEED polygon designed by the user
// The polygon is designed as a closed loop (last point == first point)
// and the angles are assumed to be equal (square, hexagon, etc).
// The corrections (deltaX(i,j) and deltaY(i,j)) are saved into the SIS folder as two image files.
// The corrected image is saved into the current folder with the same name + "_dc"
//--------------------------------------------------------------------------------------
function LEEDCor_determineDistortion()
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	String winNameStr=SIS_getWinName()
	String imageStr=SIS_getImageStackName()
	
	//print "Name of the image is: ", imageStr, "    dimsize = ", dimsize($imageStr,2)
	String ciwStr = imageStr+"_ciw"
	if (dimsize($imageStr,2) > 1)
		NVAR layer=root:$(dfStr):layer
		make/N=(dimsize($imageStr,0),dimsize($imageStr,1)) $ciwStr
		LEEDCor_extractLayer($ciwStr, $imageStr, layer)
	else
		duplicate/o $imageStr, $ciwStr
	endif	
	
	wave tmpwY, tmpwX
	make/O/N=1 tmpwY
	make/O/N=1 tmpwX
	wave idealwY,idealwX, deltawX, deltawY, anglew
	
	variable shapeFlag=1
	Prompt shapeFlag, "Select the shape", popup "regular polygon; rectangle bcc/fcc(110);"
	doPrompt "LEED Correction" shapeFlag
	
	graphwavedraw/o/L/T tmpwY, tmpwX
	SIS_PauseForCursors(winNameStr,"Design the closed polygon")
	showTools
	hideTools
	
	print tmpwY[0], tmpwX[0]

	duplicate/o tmpwX, idealwX
	duplicate/o tmpwY, idealwY
	duplicate/o tmpwX, deltawX
	duplicate/o tmpwY, deltawY
	duplicate/o tmpwY, anglew
	variable nopts = dimsize(tmpwX,0)
	print "No of points selected: ",num2str(nopts)

	if ((shapeFlag == 2) && (nopts != 5))
		Print "!!!! For the rectangular (110) pattern, select only the 4 corners  !!!!!"
		Print "Proceeding as if you selected a regular polygon with ", num2str(nopts-1), " corners."
		shapeFlag = 1
	endif

	variable i=0
	variable j=0
	
	// Print "Find the peaks around the marked positions:"
	i=0
	do
		print tmpwX[i], tmpwY[i]
		CurveFit/NTHR=0/Q Gauss2D  $ciwStr[tmpwX[i]-10,tmpwX[i]+10][tmpwY[i]-10,tmpwY[i]+10] /D
		tmpwX[i] = K2
		tmpwY[i] = K4
		i=i+1
	while (i<nopts)
	
	variable centerX=0, centerY=0
	variable avgRadius = 0
	variable preferedAngle=0
	
	// find the center
	i=0
	do
		centerX += tmpwX[i] / (nopts-1)
		centerY += tmpwY[i] / (nopts-1)
		i=i+1
	while (i<nopts-1)
	
	// find the radius
	i=0
	do
		avgRadius += sqrt( (tmpwX[i] - centerX)^2 + (tmpwY[i] - centerY)^2 ) / (nopts-1)
		i=i+1
	while (i<nopts-1)
	
	print "Average center (X,Y) = ", num2str(centerX), ", ", num2str(centerY), "   radius = ", num2str(avgRadius)

	// find the ideal positions for a polygon, and find the correction values at the edges of the polygon
	variable zeroAngle = atan((tmpwY[0]-centerY)/(tmpwX[0]-centerX))
	if (tmpwX[0] < centerX)
		zeroAngle += pi
	endif
	variable tempoAngle
	
	if (shapeFlag == 1)
		// regular polygon
		i=0
		zeroAngle = 0
		do
			anglew[i] = atan((tmpwY[i]-centerY)/(tmpwX[i]-centerX))
			if (tmpwX[i] < centerX)
				anglew[i] += pi
			endif
			if ((i>0) && (anglew[i] < anglew[i-1]))
				anglew[i] += 2*pi
			endif
			zeroAngle += (anglew[i] - i*2*pi/(nopts-1)) / (nopts-1)
			i=i+1
		while (i<nopts-1)
		i=0
		do
			tempoAngle = atan((tmpwY[i]-centerY)/(tmpwX[i]-centerX))
			if (tmpwX[i] < centerX)
				tempoAngle += pi
			endif
			anglew[i] = zeroAngle + i*2*pi/(nopts-1)
			idealwX[i] = centerX + cos(zeroAngle + i*2*pi/(nopts-1)) * avgRadius
			idealwY[i] = centerY + sin(zeroAngle + i*2*pi/(nopts-1)) * avgRadius
			deltawX[i] = - (idealwX[i] - tmpwX[i])
			deltawY[i] = - (idealwY[i] - tmpwY[i])
			//print "Point ", num2str(i+1), "  (x,y) = (", num2str(tmpwX[i]), ", ", num2str(tmpwY[i]), "),    ideal (x,y) = (", num2str(idealwX[i]), ", ", num2str(idealwY[i]), ")"
			print "Point ", num2str(i+1), "  delta(x,y) = (", num2str(deltawX[i]), ", ", num2str(deltawY[i]), ")"
			//print "Point ", num2str(i+1), "  angleCalc = ", num2str(anglew[i]*180/pi), "     angleMeas = ", num2str(tempoAngle*180/pi)
			i=i+1
		while (i<nopts)
	else
		// (110) rectangle
		variable delta110Angle
		if ( ((tmpwX[0] - tmpwX[1])^2 + (tmpwY[0] - tmpwY[1])^2) <  ((tmpwX[1] - tmpwX[2])^2 + (tmpwY[1] - tmpwY[2])^2))
			delta110Angle = 2*atan(1/sqrt(2))
		else
			delta110Angle = 2*atan(sqrt(2))
		endif
		tempoAngle = delta110angle
		
		i=0
		zeroAngle = 0
		do
			anglew[i] = atan((tmpwY[i]-centerY)/(tmpwX[i]-centerX))
			if (tmpwX[i] < centerX)
				anglew[i] += pi
			endif
			if ((i>0) && (anglew[i] < anglew[i-1]))
				anglew[i] += 2*pi
			endif
			if (i>0)
				anglew[i] -= tempoAngle
				delta110Angle = pi - delta110Angle
				tempoAngle += (delta110angle)
			endif
			zeroAngle += (anglew[i]) / (nopts-1)
			i=i+1
		while (i<nopts-1)
		
		if ( ((tmpwX[0] - tmpwX[1])^2 + (tmpwY[0] - tmpwY[1])^2) <  ((tmpwX[1] - tmpwX[2])^2 + (tmpwY[1] - tmpwY[2])^2)) 
			delta110Angle = 2*atan(1/sqrt(2))
		else
			delta110Angle = 2*atan(sqrt(2))
		endif
		
		anglew[0] = zeroAngle
		idealwX[0] = centerX + cos(zeroAngle) * avgRadius
		idealwY[0] = centerY + sin(zeroAngle) * avgRadius
		deltawX[0] = - (idealwX[0] - tmpwX[0])
		deltawY[0] = - (idealwY[0] - tmpwY[0])
		//print "Point ", num2str(i+1), "  (x,y) = (", num2str(tmpwX[i]), ", ", num2str(tmpwY[i]), "),    ideal (x,y) = (", num2str(idealwX[i]), ", ", num2str(idealwY[i]), ")"
		print "Point ", num2str(1), "  delta(x,y) = (", num2str(deltawX[0]), ", ", num2str(deltawY[0]), ")"
		//print "Point ", num2str(i+1), "  angle = ", num2str(anglew[i]*180/pi)
		i=1
		do
			anglew[i] = anglew[i-1] + delta110Angle
			delta110Angle = pi - delta110Angle
			idealwX[i] = centerX + cos(anglew[i]) * avgRadius
			idealwY[i] = centerY + sin(anglew[i]) * avgRadius
			deltawX[i] = - (idealwX[i] - tmpwX[i])
			deltawY[i] = - (idealwY[i] - tmpwY[i])
			print "Point ", num2str(i+1), "  (x,y) = (", num2str(tmpwX[i]), ", ", num2str(tmpwY[i]), "),    ideal (x,y) = (", num2str(idealwX[i]), ", ", num2str(idealwY[i]), ")"
			print "Point ", num2str(i+1), "  delta(x,y) = (", num2str(deltawX[i]), ", ", num2str(deltawY[i]), ")"
			print "Point ", num2str(i+1), "  angle = ", num2str(anglew[i]*180/pi)
			i=i+1
		while (i<nopts)		
	endif

	RemoveFromGraph tmpwY
	RemoveContour $"fit_"+imageStr
	
	// create the 2D correction waves
	wave corrX2d, corrY2d
	variable noptsX, noptsY, noptsZ
	variable tmpAngle, tmpRadius
	String correctedStr = imageStr+"_dc"
	duplicate/o $ciwStr, corrX2d
	duplicate/o $ciwStr, corrY2d
	duplicate/o $imageStr, $correctedStr
	wave corrected2d = $correctedStr
	noptsX = dimsize(corrX2d,0)
	noptsY = dimsize(corrX2d,1)
	noptsZ = dimsize(corrected2d,2)

	// fill in the X corrections
	wave w_coef
	make/n=5/o w_coef
	//K2 = 1
	//CurveFit/H="0010"/NTHR=0/Q sin  deltawX /X=anglew /D
	w_coef[0]= 0
	w_coef[1] = 1
	w_coef[2] = 0.1
	w_coef[3] = 1
	w_coef[4] = 0.1
	FuncFit/Q/NTHR=0 fitAzimuthal w_coef deltawX /X=anglew /D
	 i=0
	 do
	 	j=0
	 	do
	 		tmpAngle =  atan((j-centerY)/(i-centerX))
	 		if (i<centerX)
	 			tmpAngle = tmpAngle + pi
	 		endif
	 		tmpRadius = sqrt( (j-centerY)^2 + (i-centerX)^2 )
	 		//corrX2d[i][j] = (tmpRadius / avgRadius)^2 * (K0 + K1 * sin(K2*tmpAngle + K3))
	 		corrX2d[i][j] = (tmpRadius / avgRadius)^2 * fitAzimuthal(w_coef, tmpAngle)
	 		j=j+1
	 	while (j<noptsY)
	 	i=i+1
	 while (i<noptsX)

	// fill in the Y corrections
	//K2 = 1
	//CurveFit/H="0010"/NTHR=0/Q sin  deltawY /X=anglew /D
	w_coef[0]= 0
	w_coef[1] = 1
	w_coef[2] = 0.1
	w_coef[3] = 1
	w_coef[4] = 0.1
	FuncFit/Q/NTHR=0 fitAzimuthal w_coef deltawY /X=anglew /D
	print num2str(w_coef[0]), num2str(w_coef[1]), num2str(w_coef[2]), num2str(w_coef[3]), num2str(w_coef[4])
	i=0
	do
		j=0
		do
	 		tmpAngle =  atan((j-centerY)/(i-centerX))
	 		if (i<centerX)
	 			tmpAngle = tmpAngle + pi
	 		endif
	 		tmpRadius = sqrt( (j-centerY)^2 + (i-centerX)^2 )
	 		//corrY2d[i][j] = (tmpRadius / avgRadius)^2 * (K0 + K1 * sin(K2*tmpAngle + K3))
	 		corrY2d[i][j] = (tmpRadius / avgRadius)^2 * fitAzimuthal(w_coef, tmpAngle)
	 		j=j+1
	 	while (j<noptsY)
	 	i=i+1
	 while (i<noptsX)

	// save the correction images into the SIS folder
	duplicate/o corrX2d,  root:SIS:corrX2d
	duplicate/o corrY2d,  root:SIS:corrY2d
	
	// find the distortion corrected image from the displacements
	variable targetX, targetY, tmpSum, k
	wave imagew = $imageStr
	corrected2d = 0
	k=0
	do
		i=0
		do
			j=0
			do
	 			targetX = i + corrX2d[i][j]
	 			targetY = j + corrY2d[i][j]
	 			if ((targetX>0) && (targetX<noptsX-1) && (targetY>0) && (targetY<noptsY-1) )
	 				corrected2d[i][j][k] = imagew[trunc(targetX)][trunc(targetY)][k] / sqrt((targetX - trunc(targetX))^2 + (targetY - trunc(targetY))^2 + 1e-6)
	 				corrected2d[i][j][k] += imagew[trunc(targetX)+1][trunc(targetY)][k] / sqrt((trunc(targetX)+1-targetX)^2 + (targetY - trunc(targetY))^2 + 1e-6)
	 				corrected2d[i][j][k] += imagew[trunc(targetX)][trunc(targetY)+1][k] / sqrt((targetX - trunc(targetX))^2 + (trunc(targetY)+1-targetY)^2 + 1e-6)
	 				corrected2d[i][j][k] += imagew[trunc(targetX)+1][trunc(targetY)+1][k] / sqrt((trunc(targetX)+1-targetX)^2 + (trunc(targetY)+1-targetY)^2 + 1e-6)
	 				tmpSum = 1 / sqrt((targetX - trunc(targetX))^2 + (targetY - trunc(targetY))^2 + 1e-6)
	 				tmpSum += 1 / sqrt((trunc(targetX)+1-targetX)^2 + (targetY - trunc(targetY))^2 + 1e-6)
	 				tmpSum += 1 / sqrt((targetX - trunc(targetX))^2 + (trunc(targetY)+1-targetY)^2 + 1e-6)
	 				tmpSum += 1 / sqrt((trunc(targetX)+1-targetX)^2 + (trunc(targetY)+1-targetY)^2 + 1e-6)
	 				corrected2d[i][j][k] /= tmpSum
	 			else
	 				corrected2d[i][j][k] = 0
	 			endif
	 			j=j+1
		 	while (j<noptsY)
		 	i=i+1
		 while (i<noptsX)
		 print k
		 k=k+1
	while (k<noptsZ)
	
end

//---------------------------------------------------------------------------------
// Correct the distortions of a LEED pattern. Obtain the displacements
// for each coordinate from the  already existing correction images
// which should be present in the SIS folder.
//---------------------------------------------------------------------------------
function LEEDCor_correctDistortion()
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	String winNameStr=SIS_getWinName()
	String imageStr=SIS_getImageStackName()
	
	print "Name of the image is: ", imageStr
	wave imagew = $imageStr
	
	// create the 2D correction waves
	wave corrX2d = $"root:SIS:corrX2d"
	wave corrY2d = $"root:SIS:corrY2d"
	variable abortFlag = 1
	variable noptsX, noptsY
	if (waveExists(corrX2d) && waveExists(corrX2d))
		noptsX = dimsize(corrX2d,0)
		noptsY = dimsize(corrX2d,1)
		if ((noPtsX == dimsize(imagew,0)) && (noPtsY == dimsize(imagew,1)) )
			abortFlag = 0
		endif
	endif
	
	if (!abortFlag)
		
		String correctedStr = imageStr+"_dc"
		duplicate/o $imageStr, $correctedStr
		wave corrected2d = $correctedStr

		// find the distortion corrected image from the displacements
		variable i=0
		variable j=0
		variable targetX, targetY, tmpSum
		corrected2d = 0
		i=0
		do
			j=0
			do
	 			targetX = i + corrX2d[i][j]
	 			targetY = j + corrY2d[i][j]
	 			if ((targetX>0) && (targetX<noptsX-1) && (targetY>0) && (targetY<noptsY-1) )
	 				corrected2d[i][j] = imagew[trunc(targetX)][trunc(targetY)] / sqrt((targetX - trunc(targetX))^2 + (targetY - trunc(targetY))^2 + 1e-6)
	 				corrected2d[i][j] += imagew[trunc(targetX)+1][trunc(targetY)] / sqrt((trunc(targetX)+1-targetX)^2 + (targetY - trunc(targetY))^2 + 1e-6)
	 				corrected2d[i][j] += imagew[trunc(targetX)][trunc(targetY)+1] / sqrt((targetX - trunc(targetX))^2 + (trunc(targetY)+1-targetY)^2 + 1e-6)
	 				corrected2d[i][j] += imagew[trunc(targetX)+1][trunc(targetY)+1] / sqrt((trunc(targetX)+1-targetX)^2 + (trunc(targetY)+1-targetY)^2 + 1e-6)
	 				tmpSum = 1 / sqrt((targetX - trunc(targetX))^2 + (targetY - trunc(targetY))^2 + 1e-6)
	 				tmpSum += 1 / sqrt((trunc(targetX)+1-targetX)^2 + (targetY - trunc(targetY))^2 + 1e-6)
	 				tmpSum += 1 / sqrt((targetX - trunc(targetX))^2 + (trunc(targetY)+1-targetY)^2 + 1e-6)
	 				tmpSum += 1 / sqrt((trunc(targetX)+1-targetX)^2 + (trunc(targetY)+1-targetY)^2 + 1e-6)
	 				corrected2d[i][j] /= tmpSum
	 			else
	 				corrected2d[i][j] = 0
	 			endif
	 			j=j+1
	 		while (j<noptsY)
	 		i=i+1
	 	while (i<noptsX)
	 
	 else
	 	print "Function return with an abort!"
	 endif
	
end

//---------------------------------------------------------------------------------------------
Function LEEDCor_extractLayer(curw, stackw, layernum)
	wave curw
	wave stackw
	variable layernum
	
	variable pnum = dimsize(stackw,0)
	variable qnum = dimsize(stackw,1)
	if (layernum < dimsize(stackw,2))
		variable i,j
		i=0
		do
			j=0
			do
				curw[i][j] = stackw[i][j][layernum]
				j=j+1
			while (j<qnum)
			i=i+1
		while (i<pnum)
	endif

end

//-------------------------------------------------------------
//---- Here is the actual fitting function --------------------------------
//---------------------------------------------------------------------------------
Function fitAzimuthal(w,x) : FitFunc
	WAVE w
	Variable x
	
	//CurveFitDialog/ Equation:
	//CurveFitDialog/ f(x) = y0 + A*sin(x + phi1) + B*sin(2*x + phi2)
	//CurveFitDialog/ End of Equation
	//CurveFitDialog/ Independent Variables 1
	//CurveFitDialog/ x
	//CurveFitDialog/ Coefficients 5
	//CurveFitDialog/ w[0] = y0
	//CurveFitDialog/ w[1] = A
	//CurveFitDialog/ w[2] = phi1
	//CurveFitDialog/ w[3] = B
	//CurveFitDialog/ w[4] = phi2

	variable tmpSum
	// background: polynomial to second order
	tmpSum=w[0] + w[1] * sin(x + w[2]) + w[3] * sin(2*x + w[4])
	return  tmpSum

End
